@@ -332,25 +332,6 @@ pub fn einsum_nest_optimize(einsum: &Einsum, context: &mut OptimizationContext)
     Some(einsum_nest_path(einsum, path))
 }
 
-#[pyfunction]
-#[pyo3(name = "deep_optimize_einsums")]
-pub fn deep_optimize_einsums_py(circ: CircuitRc) -> CircuitRc {
-    deep_optimize_einsums(circ, &mut Default::default())
-}
-
-pub fn deep_optimize_einsums(circ: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
-    deep_map_op_context(
-        &circ,
-        &mut |x, c| match x {
-            Circuit::Einsum(einsum) => einsum_nest_optimize(&einsum, c).map(|z| z.rc()),
-            _ => None,
-        },
-        context,
-        &mut Default::default(),
-    )
-    .unwrap_or(circ)
-}
-
 /// corresponds to python function `construct_diag`
 pub fn expand_maybe_diag(
     node: &CircuitRc,
