@@ -227,7 +227,10 @@ pub trait CircuitNode: CircuitNodePrivate {
         self.info_mut().shape = shape.clone(); // set shape so methods to compute other info can use it
 
         let mut hasher = self.compute_hash();
-        let named_axes = if self.children().any(|x| !x.info().named_axes.is_empty()) {
+        let named_axes = if self
+            .children()
+            .any(|x| !x.info().named_axes.iter().all(|x| x.is_none()))
+        {
             self.compute_named_axes()
         } else {
             vec![None; shape.len()]
@@ -796,7 +799,13 @@ pub enum CircuitConstructionError {
     IndexRankTooHigh { index_rank: usize, node_rank: usize },
 
     #[error("Index out of bounds, {index:?} {shape:?}")]
-    IndexOutOfBounds { index: TensorIndex, shape: Shape },
+    IndexOutOfBounds {
+        index: TensorIndex,
+        shape: Shape,
+        at: usize,
+        axis: TensorAxisIndex,
+        l: usize,
+    },
 
     #[error("Scatter shape wrong, index: {index_shape:?} child: {shape:?} {index:?}")]
     ScatterShapeWrong {
