@@ -2,7 +2,9 @@ use smallvec::Array;
 use uuid::Uuid;
 
 use super::{OpSize, RearrangeSpec, TensorAxisIndex, TensorIndex};
-use crate::{all_imports::TorchDeviceDtypeOp, smallvec::Sv, tensor_util::Shape};
+use crate::{
+    all_imports::TorchDeviceDtypeOp, opt_einsum::EinsumSpec, smallvec::Sv, tensor_util::Shape,
+};
 
 pub trait RustRepr {
     /// Serialize objects to Rust expression source code
@@ -82,6 +84,17 @@ impl RustRepr for RearrangeSpec {
     }
 }
 
+impl RustRepr for EinsumSpec {
+    fn repr(&self) -> String {
+        format!(
+            "EinsumSpec{{input_ints:{}, output_ints:{}, int_sizes:{}}}",
+            self.input_ints.repr(),
+            self.output_ints.repr(),
+            self.int_sizes.repr()
+        )
+    }
+}
+
 impl RustRepr for TorchDeviceDtypeOp {
     fn repr(&self) -> String {
         if let (None, None) = (&self.device, &self.dtype) {
