@@ -423,19 +423,47 @@ impl TensorIndex {
         })
     }
 
+    pub fn validate(&self, shape: &Shape) -> Result<(), CircuitConstructionError> {
+        let get_err = |at| {
+            Err(CircuitConstructionError::IndexOutOfBounds {
+                index: self.clone(),
+                shape: shape.clone(),
+                at,
+                axis: self.0[at].clone(),
+                l: shape[at],
+            })
+        };
+
+        let check = |at: usize, i: Option<i64>, l: usize, is_slice: bool| {
+            if let Some(i) = i {
+                let end_b = if is_slice { l + 1 } else { l };
+                if i >= end_b as i64 || i < -(l as i64) {
+                    return get_err(at);
+                }
+            }
+            Ok(())
+        };
+
+        for (at, (idx, &l)) in zip(&self.0, shape).enumerate() {
+            match idx {
+                &TensorAxisIndex::Single(i) => check(at, Some(i), l, false)?,
+                &TensorAxisIndex::Slice(Slice { start, stop }) => {
+                    check(at, start, l, true)?;
+                    check(at, stop, l, true)?;
+                }
+                _ => (),
+            }
+        }
+
+        Ok(())
+    }
+
     pub fn canonicalize(&self, shape: &Shape) -> TensorIndex {
+        self.validate(shape).expect("invalid tensor index");
         TensorIndex(
             zip(&self.0, shape)
                 .map(|(idx, &l)| match idx {
-                    TensorAxisIndex::Single(i) => {
-                        assert!(
-                            *i < l as i64 && *i >= -(l as i64),
-                            "single oob idx {} length {}",
-                            i,
-                            l
-                        );
-                        TensorAxisIndex::Single((*i + l as i64) % l as i64)
-                    }
+                    TensorAxisIndex::Single(i) => TensorAxisIndex::Single((*i + l as i64) % l as i64),
                     TensorAxisIndex::Tensor(t) => TensorAxisIndex::Tensor(t.clone()), // not bothering to canon tensor for now
                     TensorAxisIndex::Slice(sl) => TensorAxisIndex::Slice(sl.canonicalize(l)),
                 })
